#include "m68k.cc"

#include <sys/time.h>

cxxrtl_design::p_fx68k m68k;

static void up()
{
  m68k.p_enPhi1.set(true);
  m68k.step();

  m68k.p_clk.set(true);
  m68k.step();

  m68k.p_enPhi1.set(false);
  m68k.p_clk.set(false);
  m68k.step();
}

static void down()
{
  m68k.p_enPhi2.set(true);
  m68k.step();

  m68k.p_clk.set(true);
  m68k.step();

  m68k.p_enPhi2.set(false);
  m68k.p_clk.set(false);
  m68k.step();
}

static void default_signals()
{
  m68k.p_DTACKn.set(true);
  m68k.p_VPAn.set(true);
  m68k.p_BERRn.set(true);
  m68k.p_BRn.set(true);
  m68k.p_BGACKn.set(true);
  m68k.p_IPL0n.set(true);
  m68k.p_IPL1n.set(true);
  m68k.p_IPL2n.set(true);
  m68k.p_iEdb.set<uint16_t>(0x0000);
  m68k.p_HALTn.set(true);

  m68k.p_enPhi1.set(false);
  m68k.p_enPhi2.set(false);
  m68k.p_clk.set(false);

  m68k.p_extReset.set(true);
  m68k.p_pwrUp.set(true);
  m68k.step();

  up();
  down();

  m68k.p_extReset.set(false);
  m68k.p_pwrUp.set(false);

  up();
  down();
}

static uint64_t cur_tick;
static int state;

enum {
  B_IDLE,
  B_IN_READ
};

static int do_read(int adr, int fc, int uds, int lds)
{
  switch(adr) {
  case 0x000000: return 0x1234;
  case 0x000002: return 0x5678;
  case 0x000004: return 0x9abc;
  case 0x000006: return 0xdef0;
  default: return 0x4e71;
  }
}

static void state_check()
{
  switch(state) {
  case B_IDLE:
    if(!m68k.p_ASn.get<bool>()) {
      int fc = (m68k.p_FC2.get<bool>() << 2) | (m68k.p_FC1.get<bool>() << 1) | m68k.p_FC0.get<bool>();
      int adr = m68k.p_eab.get<uint32_t>() << 1;
      int rw = m68k.p_eRWn.get<bool>();
      int uds = m68k.p_UDSn.get<bool>();
      int lds = m68k.p_LDSn.get<bool>();
	
      //      fprintf(stderr, "%ld: fc=%d adr=%06x rw=%d uds=%d lds=%d\n", cur_tick, fc, adr, rw, uds, lds);

      if(rw) {
	state = B_IN_READ;	
	m68k.p_iEdb.set<uint16_t>(do_read(adr, fc, uds, lds));
	m68k.p_DTACKn.set(false);
      }
    }
    break;

  case B_IN_READ:
    if(m68k.p_ASn.get<bool>()) {
      state = B_IDLE;
      m68k.p_DTACKn.set(true);
    }
    break;
  }
}


static void run(int ticks)
{
  for(int i=0; i != ticks; i++) {
    up();
    state_check();
    down();
    state_check();
    cur_tick++;
  }
}

static constexpr int count = 1000000;

int main()
{
  default_signals();
  up();
  down();

  timeval ts, te;

  cur_tick = 0;
  state = B_IDLE;
  gettimeofday(&ts, nullptr);
  run(count);
  gettimeofday(&te, nullptr);
  double dt = (te.tv_sec - ts.tv_sec) + 1e-6 * (te.tv_usec - ts.tv_usec);
  double freq = count / dt;

  printf("Measured frequency: %f\n", freq);

  return 0;
}
